/*
* Fabric3
* Copyright (c) 2009-2012 Metaform Systems
*
* Fabric3 is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of
* the License, or (at your option) any later version, with the
* following exception:
*
* Linking this software statically or dynamically with other
* modules is making a combined work based on this software.
* Thus, the terms and conditions of the GNU General Public
* License cover the whole combination.
*
* As a special exception, the copyright holders of this software
* give you permission to link this software with independent
* modules to produce an executable, regardless of the license
* terms of these independent modules, and to copy and distribute
* the resulting executable under terms of your choice, provided
* that you also meet, for each linked independent module, the
* terms and conditions of the license of that module. An
* independent module is a module which is not derived from or
* based on this software. If you modify this software, you may
* extend this exception to your version of the software, but
* you are not obligated to do so. If you do not wish to do so,
* delete this exception statement from your version.
*
* Fabric3 is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
*
* You should have received a copy of the
* GNU General Public License along with Fabric3.
* If not, see <http://www.gnu.org/licenses/>.
*/
package org.fabric3.spi.contribution.manifest;

import javax.xml.namespace.QName;

import org.oasisopen.sca.Constants;

import org.fabric3.spi.contribution.Export;
import org.fabric3.spi.contribution.Import;

/**
 * Represents an <code>export.java</code> entry in a contribution manifest.
 *
 * @version $Rev: 11083 $ $Date: 2012-06-21 10:38:50 +0000 (Thu, 21 Jun 2012) $
 */
public class JavaExport implements Export {
    private static final long serialVersionUID = -1362112844218693711L;

    private static final QName QNAME = new QName(Constants.SCA_NS, "export.java");

    private PackageInfo packageInfo;
    private boolean resolved;

    public JavaExport(PackageInfo packageInfo) {
        if (packageInfo == null) {
            throw new IllegalStateException("Package info cannot be null");
        }
        this.packageInfo = packageInfo;
    }

    public QName getType() {
        return QNAME;
    }

    public PackageInfo getPackageInfo() {
        return packageInfo;
    }

    public boolean match(Import imprt) {
        if (imprt instanceof JavaImport) {
            JavaImport javaImport = (JavaImport) imprt;
            if (javaImport.getPackageInfo().matches(packageInfo)) {
                return true;
            }
        }
        return false;
    }

    public boolean isResolved() {
        return resolved;
    }

    public void resolve() {
        resolved = true;
    }

    public String toString() {
        return packageInfo.toString();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        JavaExport that = (JavaExport) o;

        return !(packageInfo != null ? !packageInfo.equals(that.packageInfo) : that.packageInfo != null);

    }

    @Override
    public int hashCode() {
        return packageInfo != null ? packageInfo.hashCode() : 0;
    }
}

